def render():

    """Runs vmd and renders images

    Params:

    Returns:

    """

    # Requires:
    # Names of vmd and tcl scripts
    # Options:
    # render without opening vmd
    # open vmd before sourcing file
    raise NotImplementedError()
