#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aimsprop', 'aimsprop.iam', 'aimsprop.iam.ext', 'aimsprop.vis_aims']

package_data = \
{'': ['*'], 'aimsprop.vis_aims': ['ref/*']}

install_requires = \
['numpy', 'matplotlib']

extras_require = \
{'dev': ['black>=20.8b1',
         'flake8 >=3.8.4',
         'isort >= 5.7.0',
         'pre-commit >= 2.11.1',
         'autoflake >= 1.4'],
 'doc': ['mkdocs == 1.1.2',
         'mkdocs-material == 7.0.3',
         'mkdocstrings == 0.15.0'],
 'test': ['pytest >=6.2.2',
          'pytest-cov >=2.10.1,<3.0.0',
          'coverage >=5.3.1,<6.0']}

setup(name='aimsprop',
      version='0.2.1',
      description='A repository for the representation and manipulation of AIMS-type trajectories.',
      author='Rob Parrish',
      author_email='mtzgroupadmin@gmail.com',
      url='https://mtzgroup.github.io/aimsprop/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
